// Code generated by mkcnames -i a.out.go -o cnames.go -p loong64; DO NOT EDIT.

package loong64

// This order should be strictly consistent to that in a.out.go.
var cnames0 = []string{
	"NONE",
	"REG",
	"FREG",
	"FCSRREG",
	"FCCREG",
	"VREG",
	"XREG",
	"ARNG",
	"ELEM",
	"ZCON",
	"U1CON",
	"U2CON",
	"U3CON",
	"U4CON",
	"U5CON",
	"U6CON",
	"U7CON",
	"U8CON",
	"S5CON",
	"US12CON",
	"UU12CON",
	"S12CON",
	"U12CON",
	"12CON",
	"S13CON20_0",
	"S13CON",
	"U13CON20_0",
	"U13CON",
	"13CON",
	"U15CON",
	"U15CON20_0",
	"32CON20_0",
	"32CON",
	"DCON20S_0",
	"DCON12_0",
	"DCON32_0",
	"DCON12_20S",
	"DCON20S_20",
	"DCON32_20",
	"DCON12_12S",
	"DCON20S_12S",
	"DCON32_12S",
	"DCON12_32S",
	"DCON20S_32",
	"DCON12_12U",
	"DCON20S_12U",
	"DCON32_12U",
	"DCON",
	"SACON",
	"LACON",
	"DACON",
	"EXTADDR",
	"BRAN",
	"SAUTO",
	"LAUTO",
	"ZOREG",
	"SOREG",
	"LOREG",
	"ROFF",
	"ADDR",
	"TLS_LE",
	"TLS_IE",
	"GOTADDR",
	"TEXTSIZE",
	"GOK",
	"NCLASS",
}
