// Copyright 2025 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package strconv_test

import . "strconv"

var (
	baseError    = BaseError
	bitSizeError = BitSizeError
)
