// Code generated by go tool dist; DO NOT EDIT.

package buildcfg

import "runtime"

const DefaultGO386 = `sse2`
const DefaultGOAMD64 = `v1`
const DefaultGOARM = `7`
const DefaultGOARM64 = `v8.0`
const DefaultGOMIPS = `hardfloat`
const DefaultGOMIPS64 = `hardfloat`
const DefaultGOPPC64 = `power8`
const DefaultGORISCV64 = `rva20u64`
const defaultGOEXPERIMENT = ``
const defaultGO_EXTLINK_ENABLED = ``
const defaultGO_LDSO = ``
const version = `go1.26.2`
const defaultGOOS = runtime.GOOS
const defaultGOARCH = runtime.GOARCH
const DefaultGOFIPS140 = `off`
const DefaultCGO_ENABLED = ""
