// Copyright 2025 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package issue62640

type E struct{}

// F should be hidden within S because of the S.F field.
func (E) F() {}

type S struct {
	E
	F int
}
